@@ -1,12 +1,12 @@
-/* $Id: search.c,v 1.197.2.6 2007/10/11 05:01:31 dolorous Exp $ */
+/* $Id: search.c,v 1.193 2006/12/13 00:27:45 dolorous Exp $ */
 /**************************************************************************
  *   search.c                                                             *
  *                                                                        *
- *   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007   *
- *   Free Software Foundation, Inc.                                       *
+ *   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004 Chris Allegretta    *
+ *   Copyright (C) 2005, 2006 David Lawrence Ramsey                       *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
- *   the Free Software Foundation; either version 3, or (at your option)  *
+ *   the Free Software Foundation; either version 2, or (at your option)  *
  *   any later version.                                                   *
  *                                                                        *
  *   This program is distributed in the hope that it will be useful, but  *
@@ -39,20 +39,21 @@ static bool history_changed = FALSE;
 static bool regexp_compiled = FALSE;
 	/* Have we compiled any regular expressions? */
 
-/* Compile the regular expression regexp to see if it's valid.  Return
- * TRUE if it is, or FALSE otherwise. */
-bool regexp_init(const char *regexp)
+/* Regular expression helper functions. */
+
+/* Compile the given regular expression.  Return value 0 means the
+ * expression was invalid, and we wrote an error message on the status
+ * bar.  Return value 1 means success. */
+int regexp_init(const char *regexp)
 {
-    int rc;
-
-    assert(!regexp_compiled);
-
-    rc = regcomp(&search_regexp, regexp, REG_EXTENDED
+    int rc = regcomp(&search_regexp, regexp, REG_EXTENDED
 #ifndef NANO_TINY
 	| (ISSET(CASE_SENSITIVE) ? 0 : REG_ICASE)
 #endif
 	);
 
+    assert(!regexp_compiled);
+
     if (rc != 0) {
 	size_t len = regerror(rc, &search_regexp, NULL, 0);
 	char *str = charalloc(len);
@@ -60,13 +61,11 @@ bool regexp_init(const char *regexp)
 	regerror(rc, &search_regexp, str, len);
 	statusbar(_("Bad regex \"%s\": %s"), regexp, str);
 	free(str);
-
-	return FALSE;
+	return 0;
     }
 
     regexp_compiled = TRUE;
-
-    return TRUE;
+    return 1;
 }
 
 /* Decompile the compiled regular expression we used in the last
@@ -86,7 +85,7 @@ void not_found_msg(const char *str)
 {
     char *disp;
     int numchars;
-
+ 
     assert(str != NULL);
 
     disp = display_string(str, 0, (COLS / 2) + 1, FALSE);
@@ -208,8 +207,8 @@ int search_init(bool replacing, bool use_answer)
     backupstring = NULL;
 
     /* Cancel any search, or just return with no previous search. */
-    if (i == -1 || (i < 0 && *last_search == '\0') || (!replacing &&
-	i == 0 && *answer == '\0')) {
+    if (i == -1 || (i < 0 && last_search[0] == '\0') ||
+	    (!replacing && i == 0 && answer[0] == '\0')) {
 	statusbar(_("Cancelled"));
 	return -1;
     } else {
@@ -219,8 +218,9 @@ int search_init(bool replacing, bool use_answer)
 #ifdef HAVE_REGEX_H
 		/* Use last_search if answer is an empty string, or
 		 * answer if it isn't. */
-		if (ISSET(USE_REGEXP) && !regexp_init((i == -2) ?
-			last_search : answer))
+		if (ISSET(USE_REGEXP) &&
+			regexp_init((i == -2) ? last_search :
+			answer) == 0)
 		    return -1;
 #endif
 		break;
@@ -444,7 +444,7 @@ void do_search(void)
 	return;
 
     /* If answer is now "", copy last_search into answer. */
-    if (*answer == '\0')
+    if (answer[0] == '\0')
 	answer = mallocstrcpy(answer, last_search);
     else
 	last_search = mallocstrcpy(last_search, answer);
@@ -511,7 +511,7 @@ void do_research(void)
     if (last_search[0] != '\0') {
 #ifdef HAVE_REGEX_H
 	/* Since answer is "", use last_search! */
-	if (ISSET(USE_REGEXP) && !regexp_init(last_search))
+	if (ISSET(USE_REGEXP) && regexp_init(last_search) == 0)
 	    return;
 #endif
 
@@ -827,8 +827,7 @@ ssize_t do_replace_loop(
 		openfile->current_x += match_len + length_change - 1;
 
 	    /* Cleanup. */
-	    openfile->totsize += mbstrlen(copy) -
-		mbstrlen(openfile->current->data);
+	    openfile->totsize += length_change;
 	    free(openfile->current->data);
 	    openfile->current->data = copy;
 
@@ -978,7 +977,7 @@ void do_gotolinecolumn(ssize_t line, ssize_t column, bool use_answer,
     if (interactive) {
 	char *ans = mallocstrcpy(NULL, answer);
 
-	/* Ask for the line and column. */
+	/* Ask for it. */
 	int i = do_prompt(FALSE,
 #ifndef DISABLE_TABCOMP
 		TRUE,
@@ -1311,7 +1310,7 @@ void update_history(filestruct **h, const char *s)
     assert(hage != NULL && hbot != NULL);
 
     /* If this string is already in the history, delete it. */
-    p = find_history(*hage, *hbot, s, (size_t)-1);
+    p = find_history(*hage, *hbot, s, strlen(s));
 
     if (p != NULL) {
 	filestruct *foo, *bar;
